@@ -7,6 +7,20 @@ import (
 	"strings"
 	"time"
 
+	v1 "github.com/openshift/api/config/v1"
+	operatorv1 "github.com/openshift/api/operator/v1"
+	configinformers "github.com/openshift/client-go/config/informers/externalversions"
+	configlistersv1 "github.com/openshift/client-go/config/listers/config/v1"
+	configv1listers "github.com/openshift/client-go/config/listers/config/v1"
+	"github.com/openshift/cluster-kube-scheduler-operator/pkg/operator/operatorclient"
+	"github.com/openshift/cluster-kube-scheduler-operator/pkg/operator/v410_00_assets"
+	"github.com/openshift/cluster-kube-scheduler-operator/pkg/version"
+	"github.com/openshift/library-go/pkg/operator/events"
+	"github.com/openshift/library-go/pkg/operator/resource/resourceapply"
+	"github.com/openshift/library-go/pkg/operator/resource/resourcemerge"
+	"github.com/openshift/library-go/pkg/operator/resource/resourceread"
+	"github.com/openshift/library-go/pkg/operator/resourcesynccontroller"
+	"github.com/openshift/library-go/pkg/operator/v1helpers"
 	corev1 "k8s.io/api/core/v1"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -19,21 +33,7 @@ import (
 	corev1listers "k8s.io/client-go/listers/core/v1"
 	"k8s.io/client-go/tools/cache"
 	"k8s.io/client-go/util/workqueue"
-	"k8s.io/klog"
-
-	"github.com/openshift/api/config/v1"
-	operatorv1 "github.com/openshift/api/operator/v1"
-	configinformers "github.com/openshift/client-go/config/informers/externalversions"
-	configlistersv1 "github.com/openshift/client-go/config/listers/config/v1"
-	"github.com/openshift/cluster-kube-scheduler-operator/pkg/operator/operatorclient"
-	"github.com/openshift/cluster-kube-scheduler-operator/pkg/operator/v410_00_assets"
-	"github.com/openshift/cluster-kube-scheduler-operator/pkg/version"
-	"github.com/openshift/library-go/pkg/operator/events"
-	"github.com/openshift/library-go/pkg/operator/resource/resourceapply"
-	"github.com/openshift/library-go/pkg/operator/resource/resourcemerge"
-	"github.com/openshift/library-go/pkg/operator/resource/resourceread"
-	"github.com/openshift/library-go/pkg/operator/resourcesynccontroller"
-	"github.com/openshift/library-go/pkg/operator/v1helpers"
+	"k8s.io/klog/v2"
 )
 
 const (
@@ -49,9 +49,10 @@ type TargetConfigController struct {
 	kubeClient            kubernetes.Interface
 	eventRecorder         events.Recorder
 	configMapLister       corev1listers.ConfigMapLister
+	infrastuctureLister   configv1listers.InfrastructureLister
 	featureGateLister     configlistersv1.FeatureGateLister
-	featureGateCacheSync  cache.InformerSynced
 	configInformer        configinformers.SharedInformerFactory
+	cachesSync            []cache.InformerSynced
 	// queue only ever has one item, but it has nice error handling backoff/retry semantics
 	queue workqueue.RateLimitingInterface
 }
@@ -73,13 +74,13 @@ func NewTargetConfigController(
 		operatorImagePullSpec: operatorImagePullSpec,
 		kubeClient:            kubeClient,
 		configMapLister:       kubeInformersForNamespaces.ConfigMapLister(),
+		infrastuctureLister:   configInformer.Config().V1().Infrastructures().Lister(),
 		operatorClient:        operatorClient,
 		eventRecorder:         eventRecorder,
 		configInformer:        configInformer,
 
-		featureGateLister:    configInformer.Config().V1().FeatureGates().Lister(),
-		featureGateCacheSync: configInformer.Config().V1().FeatureGates().Informer().HasSynced,
-		queue:                workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "TargetConfigController"),
+		featureGateLister: configInformer.Config().V1().FeatureGates().Lister(),
+		queue:             workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "TargetConfigController"),
 	}
 
 	operatorConfigClient.Informer().AddEventHandler(c.eventHandler())
@@ -87,7 +88,12 @@ func NewTargetConfigController(
 	namespacedKubeInformers.Core().V1().Secrets().Informer().AddEventHandler(c.eventHandler())
 	namespacedKubeInformers.Core().V1().ServiceAccounts().Informer().AddEventHandler(c.eventHandler())
 
+	// We use infrastuctureInformer for observing load balancer URL
+	configInformer.Config().V1().Infrastructures().Informer().AddEventHandler(c.eventHandler())
+	c.cachesSync = append(c.cachesSync, configInformer.Config().V1().Infrastructures().Informer().HasSynced)
+
 	configInformer.Config().V1().FeatureGates().Informer().AddEventHandler(c.eventHandler())
+	c.cachesSync = append(c.cachesSync, configInformer.Config().V1().FeatureGates().Informer().HasSynced)
 	// we react to some config changes
 	kubeInformersForNamespaces.InformersFor(operatorclient.GlobalUserSpecifiedConfigNamespace).Core().V1().ConfigMaps().Informer().AddEventHandler(c.eventHandler())
 
@@ -134,7 +140,7 @@ func (c *TargetConfigController) Run(workers int, stopCh <-chan struct{}) {
 	klog.Infof("Starting TargetConfigController")
 	defer klog.Infof("Shutting down TargetConfigController")
 
-	if !cache.WaitForCacheSync(stopCh, c.featureGateCacheSync) {
+	if !cache.WaitForCacheSync(stopCh, c.cachesSync...) {
 		utilruntime.HandleError(fmt.Errorf("caches did not sync"))
 		return
 	}
@@ -227,7 +233,7 @@ func (c *TargetConfigController) namespaceEventHandler() cache.ResourceEventHand
 func createTargetConfigController_v311_00_to_latest(c TargetConfigController, recorder events.Recorder, operatorSpec *operatorv1.StaticPodOperatorSpec) (bool, error) {
 	errors := []error{}
 
-	_, _, err := manageKubeSchedulerConfigMap_v311_00_to_latest(c.configMapLister, c.kubeClient.CoreV1(), recorder, operatorSpec)
+	_, _, err := manageKubeSchedulerConfigMap_v311_00_to_latest(c.configMapLister, c.kubeClient.CoreV1(), recorder, operatorSpec, c.configInformer)
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "configmap", err))
 	}
@@ -239,6 +245,10 @@ func createTargetConfigController_v311_00_to_latest(c TargetConfigController, re
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "serviceaccount/localhost-recovery-client", err))
 	}
+	_, _, err = manageSchedulerKubeconfig(c.ctx, c.kubeClient.CoreV1(), c.infrastuctureLister, recorder)
+	if err != nil {
+		errors = append(errors, fmt.Errorf("%q: %v", "configmap/scheduler-kubeconfig", err))
+	}
 	_, _, err = managePod_v311_00_to_latest(c.ctx, c.kubeClient.CoreV1(), c.kubeClient.CoreV1(), recorder, operatorSpec, c.targetImagePullSpec, c.operatorImagePullSpec, c.featureGateLister, c.configInformer)
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "configmap/kube-scheduler-pod", err))
@@ -268,10 +278,34 @@ func createTargetConfigController_v311_00_to_latest(c TargetConfigController, re
 	return false, nil
 }
 
-func manageKubeSchedulerConfigMap_v311_00_to_latest(lister corev1listers.ConfigMapLister, client corev1client.ConfigMapsGetter, recorder events.Recorder, operatorSpec *operatorv1.StaticPodOperatorSpec) (*corev1.ConfigMap, bool, error) {
+func manageKubeSchedulerConfigMap_v311_00_to_latest(lister corev1listers.ConfigMapLister, client corev1client.ConfigMapsGetter, recorder events.Recorder, operatorSpec *operatorv1.StaticPodOperatorSpec, configInformer configinformers.SharedInformerFactory) (*corev1.ConfigMap, bool, error) {
 	configMap := resourceread.ReadConfigMapV1OrDie(v410_00_assets.MustAsset("v4.1.0/kube-scheduler/cm.yaml"))
-	defaultConfig := v410_00_assets.MustAsset("v4.1.0/config/defaultconfig-postbootstrap.yaml")
-	requiredConfigMap, _, err := resourcemerge.MergeConfigMap(configMap, "config.yaml", nil, defaultConfig, operatorSpec.ObservedConfig.Raw, operatorSpec.UnsupportedConfigOverrides.Raw)
+
+	var kubeSchedulerConfiguration []byte
+
+	config, err := configInformer.Config().V1().Schedulers().Lister().Get("cluster")
+	if err != nil {
+		return nil, false, err
+	}
+	// TOOD(jchaloup): remove the condition once the policy API is removed from the code
+	// Until that, ignore profiles if the policy API is used.
+	if len(config.Spec.Policy.Name) > 0 {
+		kubeSchedulerConfiguration = v410_00_assets.MustAsset("v4.1.0/config/defaultconfig-postbootstrap-lownodeutilization.yaml")
+		klog.Warningf("Setting .spec.policy is deprecated and will be removed eventually. Please use .spec.profile instead.")
+	} else {
+		switch config.Spec.Profile {
+		case v1.LowNodeUtilization, "":
+			kubeSchedulerConfiguration = v410_00_assets.MustAsset("v4.1.0/config/defaultconfig-postbootstrap-lownodeutilization.yaml")
+		case v1.HighNodeUtilization:
+			kubeSchedulerConfiguration = v410_00_assets.MustAsset("v4.1.0/config/defaultconfig-postbootstrap-highnodeutilization.yaml")
+		case v1.NoScoring:
+			kubeSchedulerConfiguration = v410_00_assets.MustAsset("v4.1.0/config/defaultconfig-postbootstrap-noscoring.yaml")
+		default:
+			return nil, false, fmt.Errorf("profile %q not recognized", config.Spec.Profile)
+		}
+	}
+
+	requiredConfigMap, _, err := resourcemerge.MergeConfigMap(configMap, "config.yaml", nil, kubeSchedulerConfiguration, operatorSpec.ObservedConfig.Raw, operatorSpec.UnsupportedConfigOverrides.Raw)
 	if err != nil {
 		return nil, false, err
 	}
@@ -374,13 +408,18 @@ func checkForFeatureGates(featureGateLister configlistersv1.FeatureGateLister) m
 		return generateFeatureGates(enabledFeatureSets, disabledFeatureSets, featureGates)
 	}
 
-	currentFeatureSetConfig := featureGateListConfig.Spec.FeatureSet
-	if featureSet, ok := v1.FeatureSets[currentFeatureSetConfig]; ok {
+	if featureGateListConfig.Spec.FeatureSet == v1.CustomNoUpgrade {
+		if featureGateListConfig.Spec.FeatureGateSelection.CustomNoUpgrade != nil {
+			enabledFeatureSets = featureGateListConfig.Spec.FeatureGateSelection.CustomNoUpgrade.Enabled
+			disabledFeatureSets = featureGateListConfig.Spec.FeatureGateSelection.CustomNoUpgrade.Disabled
+		}
+	} else if featureSet, ok := v1.FeatureSets[featureGateListConfig.Spec.FeatureSet]; ok {
 		enabledFeatureSets = featureSet.Enabled
 		disabledFeatureSets = featureSet.Disabled
 	} else {
-		klog.Infof("Invalid feature set config found in features.config.openshift.io/cluster %v. Please look at allowed features", currentFeatureSetConfig)
+		klog.Infof("Invalid feature set config found in features.config.openshift.io/cluster %v. Please look at allowed features", featureGateListConfig.Spec.FeatureSet)
 	}
+
 	return generateFeatureGates(enabledFeatureSets, disabledFeatureSets, featureGates)
 }
 
@@ -458,3 +497,25 @@ func ensureLocalhostRecoverySAToken(ctx context.Context, client corev1client.Cor
 
 	return err
 }
+
+func manageSchedulerKubeconfig(ctx context.Context, client corev1client.CoreV1Interface, infrastructureLister configv1listers.InfrastructureLister, recorder events.Recorder) (*corev1.ConfigMap, bool, error) {
+	cmString := string(v410_00_assets.MustAsset("v4.1.0/kube-scheduler/kubeconfig-cm.yaml"))
+
+	infrastructure, err := infrastructureLister.Get("cluster")
+	if err != nil {
+		return nil, false, err
+	}
+	apiServerInternalURL := infrastructure.Status.APIServerInternalURL
+	if len(apiServerInternalURL) == 0 {
+		return nil, false, fmt.Errorf("infrastucture/cluster: missing APIServerInternalURL")
+	}
+
+	for pattern, value := range map[string]string{
+		"$LB_INT_URL": apiServerInternalURL,
+	} {
+		cmString = strings.ReplaceAll(cmString, pattern, value)
+	}
+
+	requiredCM := resourceread.ReadConfigMapV1OrDie([]byte(cmString))
+	return resourceapply.ApplyConfigMap(client, recorder, requiredCM)
+}
